// Minimal tesselation shader example
// gcc whitetriangle.c ../common/*.c -lGL -o whitetriangle -I../common

#include <stdio.h>
#include "MicroGlut.h"
#include "GL_utilities.h"
// uses framework Cocoa
// uses framework OpenGL

// Globals
// Data would normally be read from files
GLfloat vertices[] = { -0.5f,-0.5f,0.0f,
						-0.5f,0.5f,0.0f,
						0.5f,-0.5f,0.0f };

// vertex array object
unsigned int vertexArrayObjID;
// Reference to shader program
GLuint shader;

// Tesselation control variables
GLint TessLevelInner = 2;
GLint TessLevelOuter1 = 2;
GLint TessLevelOuter2 = 2;
GLint TessLevelOuter3 = 2;

void init(void)
{
	// two vertex buffer objects, used for uploading the
	unsigned int vertexBufferObjID;

	dumpInfo();

	// GL inits
	glClearColor(0.2,0.2,0.5,0);

	glDisable(GL_DEPTH_TEST);

	// Load and compile shader
	shader = loadShadersGT("minimal.vs", "minimal.fs", "minimal.gs",
							"minimal.tcs", "minimal.tes");

	glUniform1i(glGetUniformLocation(shader, "TessLevelInner"), TessLevelInner);
	glUniform1i(glGetUniformLocation(shader, "TessLevelOuter1"), TessLevelOuter1);
	glUniform1i(glGetUniformLocation(shader, "TessLevelOuter2"), TessLevelOuter2);
	glUniform1i(glGetUniformLocation(shader, "TessLevelOuter3"), TessLevelOuter3);
	
	// Upload geometry to the GPU:
	
	// Allocate and activate Vertex Array Object
	glGenVertexArrays(1, &vertexArrayObjID);
	glBindVertexArray(vertexArrayObjID);
	// Allocate Vertex Buffer Objects
	glGenBuffers(1, &vertexBufferObjID);
	
	// VBO for vertex data
	glBindBuffer(GL_ARRAY_BUFFER, vertexBufferObjID);
	glBufferData(GL_ARRAY_BUFFER, 9*sizeof(GLfloat), vertices, GL_STATIC_DRAW);
	glVertexAttribPointer(glGetAttribLocation(shader, "in_Position"), 3, GL_FLOAT, GL_FALSE, 0, 0); 
	glEnableVertexAttribArray(glGetAttribLocation(shader, "in_Position"));
	
	// End of upload of geometry
	
	// Wireframe please!
	glPolygonMode(GL_FRONT, GL_LINE);
	glPolygonMode(GL_BACK, GL_LINE);
}

void display(void)
{
	printError("pre display");

	// clear the screen
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glBindVertexArray(vertexArrayObjID);	// Select VAO
	glUseProgram(shader);
		
	glPatchParameteri(GL_PATCH_DEFAULT_OUTER_LEVEL, 3);
	glPatchParameteri(GL_PATCH_DEFAULT_INNER_LEVEL, 3);
	glPatchParameteri(GL_PATCH_VERTICES, 3);
    #ifndef GL_PATCHES
        #define GL_PATCHES 0x0000000e
    #endif
	glDrawArrays(GL_PATCHES, 0, 3);
//	glDrawArrays(GL_TRIANGLES, 0, 3);

	glutSwapBuffers();
}

void key(unsigned char key, int x, int y)
{
	glUseProgram(shader);

	if (key == '+')
		TessLevelInner++;
	if (key == '-')
		if (TessLevelInner > 1)
			TessLevelInner--;
	if (key == '.')
	{
		TessLevelOuter1++;
		TessLevelOuter2++;
		TessLevelOuter3++;
	}
	if (key == ',')
	{
		if (TessLevelOuter1 > 1)
			TessLevelOuter1--;
		if (TessLevelOuter2 > 1)
			TessLevelOuter2--;
		if (TessLevelOuter3 > 1)
			TessLevelOuter3--;
	}

	if (key == '2')
		TessLevelOuter1++;
	if (key == '1')
		if (TessLevelOuter1 > 1)
			TessLevelOuter1--;
	if (key == '4')
		TessLevelOuter2++;
	if (key == '3')
		if (TessLevelOuter2 > 1)
			TessLevelOuter2--;
	if (key == '6')
		TessLevelOuter3++;
	if (key == '5')
		if (TessLevelOuter3 > 1)
			TessLevelOuter3--;

	printf("TessLevelInner = %d TessLevelOuter1 = %d TessLevelOuter2 = %d TessLevelOuter3 = %d\n", TessLevelInner, TessLevelOuter1, TessLevelOuter2, TessLevelOuter3);
	glUniform1i(glGetUniformLocation(shader, "TessLevelInner"), TessLevelInner);
	glUniform1i(glGetUniformLocation(shader, "TessLevelOuter1"), TessLevelOuter1);
	glUniform1i(glGetUniformLocation(shader, "TessLevelOuter2"), TessLevelOuter2);
	glUniform1i(glGetUniformLocation(shader, "TessLevelOuter3"), TessLevelOuter3);
	glutPostRedisplay();
}

void reshape(int x, int y)
{
	glViewport(0,0,x,y);
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	glutInitContextVersion(3, 2); // OK for GL 4 as well on Mac...
	glutCreateWindow ("GL4 triangle tesselation example +-,.123456");
	glutDisplayFunc(display);
	glutRepeatingTimer(20);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(key);
	init ();
	glutMainLoop();
}

